@@ -1,18 +1,18 @@
 #include "moar.h"
 
 /* This representation's function pointer table. */
-static const MVMREPROps this_repr;
+static const MVMREPROps CPointer_this_repr;
 
 /* Creates a new type object of this representation, and associates it with
  * the given HOW. */
 static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
-    MVMSTable *st  = MVM_gc_allocate_stable(tc, &this_repr, HOW);
+    MVMSTable *st  = MVM_gc_allocate_stable(tc, &CPointer_this_repr, HOW);
 
-    MVMROOT(tc, st, {
+    MVMROOT(tc, st) {
         MVMObject *obj = MVM_gc_allocate_type_object(tc, st);
         MVM_ASSIGN_REF(tc, &(st->header), st->WHAT, obj);
         st->size = sizeof(MVMCPointer);
-    });
+    }
 
     return st->WHAT;
 }
@@ -85,7 +85,14 @@ static void deserialize_stable_size(MVMThreadContext *tc, MVMSTable *st, MVMSeri
 
 static void deserialize(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMSerializationReader *reader) {
     MVMCPointerBody *body = (MVMCPointerBody *)data;
-    MVMint64 value = MVM_serialization_read_int(tc, reader);
+    MVMint64 value;
+
+    if (reader->root.version >= 19) {
+        value = MVM_serialization_read_int(tc, reader);
+    } else {
+        value = MVM_serialization_read_int64(tc, reader);
+    }
+
 #if MVM_PTR_SIZE == 4
     body->ptr = (void *)(MVMuint32)value;
 #else
@@ -107,10 +114,40 @@ static void serialize(MVMThreadContext *tc, MVMSTable *st, void *data, MVMSerial
 
 /* Initializes the representation. */
 const MVMREPROps * MVMCPointer_initialize(MVMThreadContext *tc) {
-    return &this_repr;
+    return &CPointer_this_repr;
 }
 
-static const MVMREPROps this_repr = {
+static void spesh(MVMThreadContext *tc, MVMSTable *st, MVMSpeshGraph *g, MVMSpeshBB *bb, MVMSpeshIns *ins) {
+    MVMuint16             opcode    = ins->info->opcode;
+    switch (opcode) {
+    case MVM_OP_unbox_i: {
+        MVMSpeshFacts *facts = MVM_spesh_get_and_use_facts(tc, g, ins->operands[1]);
+        if (facts->flags & MVM_SPESH_FACT_CONCRETE) {
+            MVMSpeshOperand *orig_operands = ins->operands;
+#if MVM_PTR_SIZE == 4
+            ins->info = MVM_op_get_op(MVM_OP_sp_get_i32);
+#else
+            ins->info = MVM_op_get_op(MVM_OP_sp_get_i64);
+#endif
+            ins->operands = MVM_spesh_alloc(tc, g, 3 * sizeof(MVMSpeshOperand));
+            ins->operands[0] = orig_operands[0];
+            ins->operands[1] = orig_operands[1];
+            ins->operands[2].lit_i16 = offsetof(MVMCPointer, body.ptr);
+            MVM_spesh_graph_add_comment(tc, g, ins, "unbox_i into %s", ins->info->name);
+        }
+        else {
+            MVM_spesh_graph_add_comment(tc, g, ins, "no devirt: not known concrete");
+        }
+        break;
+    }
+    default:
+        MVM_spesh_graph_add_comment(tc, g, ins, "reprop %s unsupported in CPointer %s",
+                ins->info->name,
+                MVM_6model_get_stable_debug_name(tc, st));
+    }
+}
+
+static const MVMREPROps CPointer_this_repr = {
     type_object_for,
     MVM_gc_allocate_object,
     NULL, /* initialize */
@@ -143,10 +180,9 @@ static const MVMREPROps this_repr = {
     NULL, /* gc_mark_repr_data */
     NULL, /* gc_free_repr_data */
     compose,
-    NULL, /* spesh */
+    spesh,
     "CPointer", /* name */
     MVM_REPR_ID_MVMCPointer,
-    0, /* refs_frames */
     NULL, /* unmanaged_size */
     NULL, /* describe_refs */
 };
