@@ -8,11 +8,11 @@ static const MVMREPROps NativeCall_this_repr;
 static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
     MVMSTable *st = MVM_gc_allocate_stable(tc, &NativeCall_this_repr, HOW);
 
-    MVMROOT(tc, st, {
+    MVMROOT(tc, st) {
         MVMObject *obj = MVM_gc_allocate_type_object(tc, st);
         MVM_ASSIGN_REF(tc, &(st->header), st->WHAT, obj);
         st->size = sizeof(MVMNativeCall);
-    });
+    }
 
     return st->WHAT;
 }
@@ -26,10 +26,15 @@ static void copy_to(MVMThreadContext *tc, MVMSTable *st, void *src, MVMObject *d
     if (src_body->lib_name) {
         dest_body->lib_name = MVM_malloc(strlen(src_body->lib_name) + 1);
         strcpy(dest_body->lib_name, src_body->lib_name);
-        dest_body->lib_handle = MVM_nativecall_load_lib(dest_body->lib_name);
+        if (src_body->entry_point)
+            dest_body->lib_handle = MVM_nativecall_load_lib(dest_body->lib_name);
     }
 
     /* Rest is just simple copying. */
+    if (src_body->sym_name) {
+        dest_body->sym_name = MVM_malloc(strlen(src_body->sym_name) + 1);
+        strcpy(dest_body->sym_name, src_body->sym_name);
+    }
     dest_body->entry_point = src_body->entry_point;
     dest_body->convention = src_body->convention;
     dest_body->num_args = src_body->num_args;
@@ -37,14 +42,21 @@ static void copy_to(MVMThreadContext *tc, MVMSTable *st, void *src, MVMObject *d
         dest_body->arg_types = MVM_malloc(sizeof(MVMint16) * (src_body->num_args ? src_body->num_args : 1));
         memcpy(dest_body->arg_types, src_body->arg_types, src_body->num_args * sizeof(MVMint16));
     }
-    dest_body->ret_type = src_body->ret_type;
-    if (src_body->jitcode) {
-        dest_body->jitcode = MVM_jit_code_copy(tc, src_body->jitcode);
+    if (src_body->arg_info) {
+        dest_body->arg_info = MVM_malloc(sizeof(MVMObject*) * (src_body->num_args ? src_body->num_args : 1));
+        memcpy(dest_body->arg_info, src_body->arg_info, src_body->num_args * sizeof(MVMObject*));
     }
-    else
-        dest_body->jitcode = NULL;
+    dest_body->ret_type = src_body->ret_type;
+#ifdef HAVE_LIBFFI
+    dest_body->ffi_ret_type = src_body->ffi_ret_type;
+    if (src_body->ffi_arg_types) {
+        dest_body->ffi_arg_types = MVM_malloc(sizeof(ffi_type *) * (dest_body->num_args ? dest_body->num_args : 1));
+        memcpy(dest_body->ffi_arg_types, src_body->ffi_arg_types, sizeof(ffi_type *) * (dest_body->num_args ? dest_body->num_args : 1));
+    }
+#endif
     dest_body->resolve_lib_name = src_body->resolve_lib_name;
     dest_body->resolve_lib_name_arg = src_body->resolve_lib_name_arg;
+    dest_body->variadic = src_body->variadic;
 }
 
 
@@ -73,7 +85,7 @@ static void serialize(MVMThreadContext *tc, MVMSTable *st, void *data, MVMSerial
     MVM_serialization_write_cstr(
         tc,
         writer,
-        !MVM_is_null(tc, body->resolve_lib_name) && !MVM_is_null(tc, body->resolve_lib_name_arg)
+        !body->resolve_lib_name && !MVM_is_null(tc, body->resolve_lib_name_arg)
             ? NULL
             : body->lib_name
     );
@@ -81,7 +93,6 @@ static void serialize(MVMThreadContext *tc, MVMSTable *st, void *data, MVMSerial
     MVM_serialization_write_int(tc, writer, body->convention);
     MVM_serialization_write_int(tc, writer, body->num_args);
     MVM_serialization_write_int(tc, writer, body->ret_type);
-    /* TODO ffi support */
     for (i = 0; i < body->num_args; i++) {
         MVM_serialization_write_int(tc, writer, body->arg_types[i]);
     }
@@ -90,6 +101,7 @@ static void serialize(MVMThreadContext *tc, MVMSTable *st, void *data, MVMSerial
     }
     MVM_serialization_write_ref(tc, writer, (MVMObject*)body->resolve_lib_name);
     MVM_serialization_write_ref(tc, writer, (MVMObject*)body->resolve_lib_name_arg);
+    MVM_serialization_write_int(tc, writer, body->variadic);
 }
 static void deserialize_stable_size(MVMThreadContext *tc, MVMSTable *st, MVMSerializationReader *reader) {
     st->size = sizeof(MVMNativeCall);
@@ -98,31 +110,29 @@ static void deserialize_stable_size(MVMThreadContext *tc, MVMSTable *st, MVMSeri
 static void deserialize(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMSerializationReader *reader) {
     MVMNativeCallBody *body = (MVMNativeCallBody *)data;
     MVMint16 i = 0;
-    if (reader->root.version >= 22) {
-        body->lib_name = MVM_serialization_read_cstr(tc, reader, NULL);
-        body->sym_name = MVM_serialization_read_cstr(tc, reader, NULL);
-        body->convention = MVM_serialization_read_int(tc, reader);
-        body->num_args = MVM_serialization_read_int(tc, reader);
-        body->ret_type = MVM_serialization_read_int(tc, reader);
-        body->arg_types = MVM_malloc(body->num_args * sizeof(MVMint16));
-        body->arg_info  = MVM_malloc(body->num_args * sizeof(MVMObject*));
-        /* TODO ffi support */
-        for (i = 0; i < body->num_args; i++) {
-            body->arg_types[i] = MVM_serialization_read_int(tc, reader);
-        }
-        for (i = 0; i < body->num_args; i++) {
-            body->arg_info[i] = MVM_serialization_read_ref(tc, reader);
-        }
-        body->resolve_lib_name = MVM_serialization_read_ref(tc, reader);
-        body->resolve_lib_name_arg = MVM_serialization_read_ref(tc, reader);
-#ifdef HAVE_LIBFFI
-        body->ffi_arg_types = MVM_malloc(sizeof(ffi_type *) * (body->num_args ? body->num_args : 1));
-        for (i = 0; i < body->num_args; i++) {
-            body->ffi_arg_types[i] = MVM_nativecall_get_ffi_type(tc, body->arg_types[i]);
-        }
-        body->ffi_ret_type = MVM_nativecall_get_ffi_type(tc, body->ret_type);
-#endif
+    body->lib_name = MVM_serialization_read_cstr(tc, reader, NULL);
+    body->sym_name = MVM_serialization_read_cstr(tc, reader, NULL);
+    body->convention = MVM_serialization_read_int(tc, reader);
+    body->num_args = MVM_serialization_read_int(tc, reader);
+    body->ret_type = MVM_serialization_read_int(tc, reader);
+    body->arg_types = body->num_args ? MVM_malloc(body->num_args * sizeof(MVMint16)) : NULL;
+    body->arg_info  = body->num_args ? MVM_malloc(body->num_args * sizeof(MVMObject*)) : NULL;
+    for (i = 0; i < body->num_args; i++) {
+        body->arg_types[i] = MVM_serialization_read_int(tc, reader);
     }
+    for (i = 0; i < body->num_args; i++) {
+        body->arg_info[i] = MVM_serialization_read_ref(tc, reader);
+    }
+    body->resolve_lib_name = (MVMCode *)MVM_serialization_read_ref(tc, reader);
+    body->resolve_lib_name_arg = MVM_serialization_read_ref(tc, reader);
+    body->variadic = MVM_serialization_read_int(tc, reader);
+#ifdef HAVE_LIBFFI
+    body->ffi_arg_types = MVM_malloc(sizeof(ffi_type *) * (body->num_args ? body->num_args : 1));
+    for (i = 0; i < body->num_args; i++) {
+        body->ffi_arg_types[i] = MVM_nativecall_get_ffi_type(tc, body->arg_types[i]);
+    }
+    body->ffi_ret_type = MVM_nativecall_get_ffi_type(tc, body->ret_type);
+#endif
 }
 
 static void gc_mark(MVMThreadContext *tc, MVMSTable *st, void *data, MVMGCWorklist *worklist) {
@@ -154,8 +164,10 @@ static void gc_cleanup(MVMThreadContext *tc, MVMSTable *st, void *data) {
         MVM_free(body->arg_types);
     if (body->arg_info)
         MVM_free(body->arg_info);
-    if (body->jitcode)
-        MVM_jit_code_destroy(tc, body->jitcode);
+#ifdef HAVE_LIBFFI
+    if (body->ffi_arg_types)
+        MVM_free(body->ffi_arg_types);
+#endif
 }
 
 static void gc_free(MVMThreadContext *tc, MVMObject *obj) {
@@ -175,7 +187,7 @@ const MVMREPROps * MVMNativeCall_initialize(MVMThreadContext *tc) {
 /* gets the setup state */
 static MVMint64 get_int(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data) {
     MVMNativeCallBody *body = (MVMNativeCallBody *)data;
-    return (body->lib_handle ? 1 + (body->jitcode ? 1 : 0) : 0);
+    return (body->lib_handle ? 1 : 0);
 }
 
 static const MVMREPROps NativeCall_this_repr = {
